function [trl, event] = trialfun_general(cfg);

% determine the trials from the general event structure, which is
% independent of the dataformat
%
% The trialdef structure can contain the following specifications
%   cfg.trialdef.eventtype  = 'string'
%   cfg.trialdef.eventvalue = number, string or list with numbers or strings
%   cfg.trialdef.prestim    = 0.300 latency in seconds (optional)
%   cfg.trialdef.poststim   = 0.700 latency in seconds (optional)
%
% If you specify
%   cfg.trialdef.eventtype  = '?'
% a list with the events in your datafile will be displayed on screen.

% read the header and event information
hdr = read_fcdc_header(cfg.headerfile);
event = read_fcdc_event(cfg.headerfile);

sel = [];
trl = [];
if strcmp(cfg.trialdef.eventtype, '?')
  % no trials should be added, show event information using subfunction and exit
  show_event(event);
  return
end
if ~isfield(cfg.trialdef, 'eventvalue')
  cfg.trialdef.eventvalue = [];
elseif ischar(cfg.trialdef.eventvalue)
  % convert single string into cell-array, otherwise the intersection does not work as intended
  cfg.trialdef.eventvalue = {cfg.trialdef.eventvalue};
end
% select all events of the specified type and with the specified value
for i=find(strcmp(cfg.trialdef.eventtype, {event.type}))
  if  isempty(cfg.trialdef.eventvalue)
    sel = [sel i];
  elseif ~isempty(intersect(event(i).value, cfg.trialdef.eventvalue))
    sel = [sel i];
  end
end
for i=sel
  % catch empty fields in the event table and interpret them meaningfully
  if isempty(event(i).offset)
    % time axis has no offset relative to the event
    event(i).offset = 0;
  end
  if isempty(event(i).duration)
    % the event does not specify a duration
    event(i).duration = 0;
  end
  % determine where the trial starts with respect to the event
  if ~isfield(cfg.trialdef, 'prestim')
    trloff = event(i).offset;
    trlbeg = event(i).sample;
  else
    % override the offset of the event
    trloff = round(-cfg.trialdef.prestim*hdr.Fs);
    % also shift the begin sample with the specified amount
    trlbeg = event(i).sample + trloff;
  end
  % determine the number of samples that has to be read (excluding the begin sample)
  if ~isfield(cfg.trialdef, 'poststim')
    trldur = max(event(i).duration - 1, 0);
  else
    trldur = round((cfg.trialdef.poststim+cfg.trialdef.prestim)*hdr.Fs) - 1;
  end
  trlend = trlbeg + trldur;
  % add the beginsample, endsample and offset of this trial to the list
  trl = [trl; [trlbeg trlend trloff]];
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that shows event table
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function show_event(event);
if isempty(event)
  fprintf('no events were found in the datafile\n');
  return
end
eventtype = unique({event.type});
Neventtype = length(eventtype);
if Neventtype==0
  fprintf('no events were found in the datafile\n');
else
  fprintf('the following events were found in the datafile\n');
  for i=1:Neventtype
    sel = find(strcmp(eventtype{i}, {event.type}));
    try
      eventvalue = unique({event(sel).value});            % cell-array with string value
      eventvalue = sprintf('''%s'' ', eventvalue{:});     % translate into a single string
    catch
      eventvalue = unique(cell2mat({event(sel).value}));  % array with numeric values or empty
      eventvalue = num2str(eventvalue);                   % translate into a single string
    end
    fprintf('event type: ''%s'' ', eventtype{i});
    fprintf('with event values: %s', eventvalue);
    fprintf('\n');
  end
end
